#!/usr/bin/env python3

import math
import secrets
import sys

wordlist = [
    'abacus',
    'abbreviate',
    'abdomen',
    'aboriginal',
    'abrupt',
    'absinthe',
    'abundant',
    'abyss',
    'academy',
    'accordion',
    'ace',
    'acidic',
    'acknowledge',
    'acne',
    'acquire',
    'acrobat',
    'activate',
    'acupuncture',
    'adaptation',
    'addition',
    'adept',
    'adhesive',
    'adjoining',
    'admire',
    'adopt',
    'adrenaline',
    'adulthood',
    'advice',
    'affinity',
    'afraid',
    'aftershock',
    'agape',
    'agenda',
    'aggressive',
    'agitate',
    'agony',
    'agriculture',
    'ahead',
    'aid',
    'ailment',
    'airship',
    'ajar',
    'alarm',
    'album',
    'alchemist',
    'alexandria',
    'alfredo',
    'algae',
    'alien',
    'alloy',
    'almond',
    'alone',
    'alphabet',
    'alternating',
    'alumni',
    'amateur',
    'amber',
    'amends',
    'amicable',
    'ammonite',
    'amnesia',
    'amok',
    'amphibious',
    'amulet',
    'anachronistic',
    'ancient',
    'androgynous',
    'angel',
    'animation',
    'ankle',
    'annual',
    'anonymous',
    'ansible',
    'anthill',
    'anvil',
    'anxiety',
    'anywhere',
    'apart',
    'ape',
    'apocalypse',
    'apple',
    'april',
    'aquarium',
    'arachnid',
    'arbiter',
    'archaeology',
    'arena',
    'arid',
    'ark',
    'armband',
    'aromantic',
    'arrow',
    'artist',
    'ascend',
    'ash',
    'asian',
    'asmr',
    'aspirations',
    'assured',
    'asterisk',
    'asymptote',
    'athletic',
    'atlas',
    'atom',
    'atrium',
    'attic',
    'auburn',
    'auction',
    'audio',
    'augmented',
    'aura',
    'auspicious',
    'autism',
    'auxiliary',
    'avantgarde',
    'avenue',
    'aviatrix',
    'award',
    'awkward',
    'awning',
    'axe',
    'axiom',
    'axle',
    'baboon',
    'bacterium',
    'badge',
    'bagpipe',
    'bakery',
    'balloon',
    'bamboo',
    'banana',
    'bartender',
    'basket',
    'bathhouse',
    'beautiful',
    'beckon',
    'bedtime',
    'beetle',
    'beginning',
    'behold',
    'belt',
    'bend',
    'berry',
    'beside',
    'bibliography',
    'bicep',
    'bidet',
    'bilingual',
    'binoculars',
    'biology',
    'birthday',
    'bismuth',
    'bitten',
    'blast',
    'blink',
    'blowfish',
    'blues',
    'boa',
    'body',
    'bog',
    'bohemian',
    'boil',
    'bok choi',
    'bolt',
    'bombastic',
    'bond',
    'bookmark',
    'bop',
    'boring',
    'bossy',
    'botanical',
    'bouncy',
    'bowling',
    'box',
    'branch',
    'breather',
    'bridge',
    'bronzed',
    'brush',
    'bubble',
    'bucket',
    'budding',
    'buff',
    'buggy',
    'building',
    'bullet',
    'bumpy',
    'bungee',
    'burger',
    'bus',
    'butch',
    'buzz',
    'cabinet',
    'cactus',
    'caddy',
    'caesar',
    'cahoots',
    'callus',
    'camping',
    'canyon',
    'cape',
    'cartwheel',
    'cast',
    'catharsis',
    'cauldron',
    'cave',
    'ceiling',
    'celiac',
    'cement',
    'century',
    'cerebral',
    'cesspool',
    'cgi',
    'chalk',
    'chess',
    'chip',
    'chlorine',
    'choice',
    'chrome',
    'churn',
    'cider',
    'cinnamon',
    'circle',
    'citadel',
    'clapping',
    'clear',
    'cling',
    'closet',
    'cluttered',
    'coax',
    'cobweb',
    'cocoa',
    'coding',
    'coelacanth',
    'coffee',
    'cog',
    'coherent',
    'coil',
    'cold',
    'communal',
    'condiment',
    'cookie',
    'copper',
    'corporeal',
    'cosplay',
    'cot',
    'couch',
    'cover',
    'cow',
    'cozy',
    'cracker',
    'creative',
    'critical',
    'crow',
    'crumb',
    'cryogenic',
    'cthulhu',
    'cube',
    'cuddle',
    'culdesac',
    'cumulus',
    'cuneiform',
    'curtain',
    'customize',
    'cuteness',
    'cyborg',
    'cyclist',
    'cynical',
    'cyrillic',
    'dab',
    'dachshund',
    'dada',
    'daffodil',
    'dagger',
    'daily',
    'dalmatian',
    'damp',
    'dancer',
    'dapper',
    'dare',
    'dating',
    'dazzle',
    'dealer',
    'debate',
    'decoration',
    'deduce',
    'deer',
    'defenestrate',
    'degrees',
    'delicacy',
    'demo',
    'density',
    'deodorant',
    'deprecate',
    'derpy',
    'destiny',
    'detergent',
    'devil',
    'dew',
    'dexterity',
    'diamond',
    'dibs',
    'dice',
    'dielectric',
    'difference',
    'dignified',
    'dilate',
    'dimension',
    'diner',
    'dionysus',
    'diplomacy',
    'direct',
    'disco',
    'ditch',
    'diverge',
    'diy',
    'documentary',
    'dodge',
    'dollop',
    'dome',
    'donate',
    'doppler',
    'dorm',
    'dot',
    'dove',
    'downstairs',
    'dragon',
    'dreaming',
    'drill',
    'drone',
    'drumbeat',
    'dryclean',
    'duality',
    'dubstep',
    'duct',
    'dud',
    'duffel',
    'dumptruck',
    'dungeon',
    'duplicate',
    'durable',
    'dust',
    'duty',
    'duvet',
    'dwell',
    'dynamic',
    'dysphoria',
    'eager',
    'early',
    'easy',
    'eat',
    'eavesdrop',
    'ebb',
    'eccentric',
    'echo',
    'eclair',
    'ecology',
    'ectoplasm',
    'eden',
    'edgy',
    'edit',
    'educational',
    'effervescent',
    'ego',
    'either',
    'elaborate',
    'elder',
    'electronic',
    'elf',
    'elixir',
    'eloquent',
    'elusive',
    'emanate',
    'embroider',
    'emergency',
    'emissary',
    'emoji',
    'empirical',
    'emu',
    'enable',
    'enby',
    'encrypted',
    'endless',
    'energy',
    'engaging',
    'enjoy',
    'enlightenment',
    'ensemble',
    'entrance',
    'enunciate',
    'envelope',
    'epaulette',
    'epilogue',
    'equator',
    'eraser',
    'ergonomic',
    'erode',
    'error',
    'erudite',
    'escape',
    'essence',
    'estrogen',
    'etcetera',
    'eternity',
    'ether',
    'etiquette',
    'etymology',
    'eucalyptus',
    'euphoria',
    'evaluate',
    'evening',
    'evidence',
    'evolution',
    'exaggerate',
    'exclaim',
    'executable',
    'exhale',
    'existential',
    'exothermic',
    'expedition',
    'extrude',
    'eyeball',
    'fabricate',
    'face',
    'faded',
    'faith',
    'falafel',
    'family',
    'fantasy',
    'faraday',
    'fashion',
    'fathom',
    'faustian',
    'favorite',
    'fax',
    'feature',
    'federated',
    'feeble',
    'felt',
    'feminist',
    'fender',
    'fermentation',
    'festival',
    'fetch',
    'feud',
    'few',
    'fez',
    'fiasco',
    'fiber',
    'fictional',
    'fidget',
    'fig',
    'filament',
    'finger',
    'fireplace',
    'fist',
    'fit',
    'fixing',
    'flare',
    'flea',
    'flirtatious',
    'float',
    'flutter',
    'fly',
    'focus',
    'foe',
    'foil',
    'folder',
    'fond',
    'footprint',
    'forgive',
    'found',
    'fox',
    'fragmented',
    'fresh',
    'friendship',
    'frog',
    'frumpy',
    'fudge',
    'fuel',
    'fugue',
    'fuji',
    'fulcrum',
    'fumble',
    'funnel',
    'furry',
    'fuse',
    'futuristic',
    'gag',
    'gains',
    'gallop',
    'game',
    'gang',
    'gap',
    'garnet',
    'gate',
    'gauntlet',
    'gazing',
    'gearshift',
    'gecko',
    'geeky',
    'gel',
    'gemini',
    'generator',
    'geode',
    'germaphobe',
    'gesture',
    'ghee',
    'ghibli',
    'ghost',
    'giantess',
    'gibberish',
    'giddy',
    'gift',
    'gigabyte',
    'gills',
    'ginger',
    'girlfriend',
    'glasses',
    'gleam',
    'glitch',
    'glowstick',
    'glum',
    'gnome',
    'goalie',
    'gobstopper',
    'goddess',
    'golden',
    'gong',
    'goose',
    'gorgon',
    'gossip',
    'goth',
    'gourd',
    'grasp',
    'greenhouse',
    'grid',
    'growing',
    'grumble',
    'guacamole',
    'guerilla',
    'guilty',
    'gullible',
    'gumball',
    'gunk',
    'gurgle',
    'gusty',
    'gutter',
    'gymnast',
    'gyroscope',
    'habit',
    'hacker',
    'hades',
    'haggard',
    'haircut',
    'halfway',
    'hamster',
    'handlebar',
    'haphazard',
    'hard',
    'hash',
    'hat',
    'haul',
    'havoc',
    'hawk',
    'haystack',
    'hazard',
    'healer',
    'hebrew',
    'hedge',
    'heist',
    'helium',
    'hemp',
    'hen',
    'heroic',
    'hetero',
    'hewn',
    'hexagon',
    'hiatus',
    'hiccup',
    'hiding',
    'highlander',
    'hijab',
    'hill',
    'himbo',
    'hint',
    'hipster',
    'history',
    'hitchhike',
    'hobbit',
    'hodgepodge',
    'hogtie',
    'hologram',
    'hometown',
    'honey',
    'hook',
    'hopeful',
    'horn',
    'hospitable',
    'hot',
    'house',
    'hover',
    'howl',
    'hubris',
    'huddle',
    'hull',
    'humanities',
    'hundred',
    'husky',
    'hut',
    'hybrid',
    'hydrate',
    'hygiene',
    'hymn',
    'hypnotist',
    'iambic',
    'ibis',
    'icecream',
    'icicle',
    'icon',
    'idealist',
    'idiom',
    'idol',
    'igloo',
    'ignite',
    'illuminate',
    'imitate',
    'immature',
    'impressive',
    'inaction',
    'inclusive',
    'independence',
    'inevitable',
    'infant',
    'ingrained',
    'inhibit',
    'initiative',
    'injured',
    'ink',
    'innocence',
    'inoculate',
    'inspired',
    'interrupt',
    'inventor',
    'inward',
    'irate',
    'iridescent',
    'irl',
    'ironic',
    'irrigate',
    'island',
    'isopropyl',
    'issue',
    'itch',
    'ivy',
    'jabberwocky',
    'jackhammer',
    'jade',
    'jamming',
    'janus',
    'jar',
    'jasmine',
    'javelin',
    'jaw',
    'jealous',
    'jerky',
    'jet',
    'jigsaw',
    'jinx',
    'jogger',
    'joint',
    'joking',
    'jorts',
    'journal',
    'joystick',
    'judgmental',
    'juggler',
    'juicy',
    'julep',
    'jump',
    'junkyard',
    'jurassic',
    'justified',
    'kale',
    'kangaroo',
    'karma',
    'kazoo',
    'keepsake',
    'kendo',
    'kerning',
    'keyhole',
    'khan',
    'kibble',
    'kick',
    'kid',
    'kilt',
    'kimchi',
    'kindness',
    'kiss',
    'kitten',
    'knack',
    'knock',
    'knuckle',
    'krill',
    'kumquat',
    'kwanzaa',
    'laboratory',
    'lacquer',
    'ladle',
    'lag',
    'laidback',
    'lakebed',
    'lamb',
    'landing',
    'lap',
    'large',
    'laser',
    'lathe',
    'laughter',
    'lavender',
    'law',
    'layer',
    'leak',
    'lectern',
    'ledge',
    'leech',
    'left',
    'legend',
    'lei',
    'lemmings',
    'lentil',
    'leopard',
    'lesbian',
    'lettuce',
    'leviathan',
    'liaison',
    'librarian',
    'lick',
    'lid',
    'lifetime',
    'lighthouse',
    'likely',
    'lily',
    'lime',
    'lint',
    'lipstick',
    'liquid',
    'list',
    'litter',
    'living',
    'llama',
    'loaf',
    'lob',
    'locator',
    'lodger',
    'logo',
    'loki',
    'lonely',
    'loop',
    'lotus',
    'love',
    'loyalty',
    'lucky',
    'lunge',
    'lymph',
    'machine',
    'madness',
    'magnet',
    'mailbox',
    'malign',
    'mammal',
    'mana',
    'margin',
    'masked',
    'mathematics',
    'maximum',
    'mayday',
    'meadow',
    'mecha',
    'media',
    'megafauna',
    'melty',
    'memory',
    'mentality',
    'mermaid',
    'mesh',
    'metropolis',
    'microscope',
    'middle',
    'migrate',
    'mill',
    'mindfully',
    'miracle',
    'misty',
    'mitten',
    'mobile',
    'modern',
    'mole',
    'moment',
    'monorail',
    'moonbeam',
    'moral',
    'mosquito',
    'moth',
    'mouse',
    'movement',
    'muddy',
    'mug',
    'multitude',
    'mummy',
    'munch',
    'mural',
    'musical',
    'mutate',
    'mystery',
    'nagging',
    'nail',
    'naked',
    'name',
    'nanotech',
    'napkin',
    'narrow',
    'nasal',
    'nature',
    'navigate',
    'neck',
    'needed',
    'negotiate',
    'neighbor',
    'neon',
    'network',
    'neuron',
    'news',
    'nice',
    'nightstand',
    'nimble',
    'nipple',
    'nitrogen',
    'noisy',
    'nomad',
    'none',
    'noon',
    'northern',
    'nosebleed',
    'notched',
    'nourishing',
    'now',
    'nuanced',
    'nuclear',
    'numerology',
    'nutshell',
    'nymph',
    'oak',
    'oatmeal',
    'obelisk',
    'objection',
    'observer',
    'occlude',
    'oceanic',
    'oddity',
    'odor',
    'offset',
    'oil',
    'oink',
    'okra',
    'old',
    'olive',
    'olympus',
    'omelette',
    'omit',
    'omniscient',
    'once',
    'onion',
    'operate',
    'opportunity',
    'optimize',
    'oral',
    'orc',
    'orderly',
    'organic',
    'origami',
    'orthogonal',
    'osmosis',
    'ostrich',
    'outside',
    'overlooking',
    'ovipositor',
    'oyster',
    'pack',
    'pad',
    'paella',
    'pager',
    'pair',
    'pale',
    'panpipes',
    'papyrus',
    'party',
    'passionate',
    'patience',
    'pause',
    'pavilion',
    'payback',
    'pear',
    'peck',
    'pedometer',
    'peek',
    'peg',
    'pellet',
    'pendulum',
    'people',
    'permanent',
    'pesky',
    'petal',
    'phalanx',
    'philosophy',
    'pho',
    'physics',
    'piano',
    'picture',
    'piercing',
    'pigment',
    'pikachu',
    'pill',
    'pimento',
    'pincushion',
    'pipe',
    'piquant',
    'pirate',
    'piston',
    'pitcher',
    'plastic',
    'please',
    'plight',
    'plot',
    'pluck',
    'poach',
    'poetry',
    'poignant',
    'poker',
    'polymorph',
    'pomegranate',
    'pony',
    'pool',
    'popcorn',
    'porthole',
    'pose',
    'pot',
    'poutine',
    'praxis',
    'pretzel',
    'princess',
    'propeller',
    'prune',
    'pseudopod',
    'psionic',
    'psychology',
    'publish',
    'puck',
    'pudding',
    'pulp',
    'pumice',
    'pun',
    'purr',
    'push',
    'pyromaniac',
    'quarry',
    'query',
    'quiet',
    'quorum',
    'rabbit',
    'racoon',
    'radio',
    'raffle',
    'raisin',
    'rake',
    'ram',
    'random',
    'rapper',
    'ratio',
    'rave',
    'ray',
    'reagent',
    'rebel',
    'recipe',
    'red',
    'reel',
    'reform',
    'regret',
    'rehabilitate',
    'reins',
    'rejoice',
    'relic',
    'remnants',
    'render',
    'reopen',
    'replenish',
    'requiem',
    'resolution',
    'return',
    'revolution',
    'rewind',
    'rhinestone',
    'rib',
    'ride',
    'rigid',
    'ringing',
    'riot',
    'ripe',
    'risky',
    'ritual',
    'river',
    'roadside',
    'robe',
    'rod',
    'rogue',
    'rollerskate',
    'roma',
    'root',
    'rose',
    'rotate',
    'roustabout',
    'rowing',
    'ruby',
    'rulebook',
    'rum',
    'rune',
    'rusty',
    'rut',
    'sacred',
    'sadness',
    'safekeeping',
    'sage',
    'sake',
    'saltshaker',
    'sanctuary',
    'sap',
    'sash',
    'satire',
    'savory',
    'saxophone',
    'saying',
    'scar',
    'schedule',
    'scientist',
    'scout',
    'scramble',
    'scuttle',
    'seagull',
    'secrecy',
    'seeking',
    'segment',
    'seitan',
    'selkie',
    'seminar',
    'sentient',
    'sequence',
    'serenity',
    'settle',
    'severe',
    'sex',
    'shade',
    'shell',
    'shipwreck',
    'shower',
    'shrink',
    'shun',
    'sibyl',
    'sickle',
    'sideways',
    'siesta',
    'sift',
    'signal',
    'silicon',
    'simulation',
    'sinkhole',
    'sisyphus',
    'sit-in',
    'six',
    'ska',
    'skeleton',
    'skit',
    'slanted',
    'sleepy',
    'slipper',
    'slouch',
    'slug',
    'smith',
    'smoke',
    'smug',
    'snake',
    'sneeze',
    'snippet',
    'snowfall',
    'snub',
    'soda',
    'softness',
    'soil',
    'solar',
    'somatic',
    'sonar',
    'sort',
    'soulful',
    'soviet',
    'space',
    'speedy',
    'spirit',
    'splinter',
    'sponge',
    'sprint',
    'squid',
    'starlight',
    'steep',
    'stilts',
    'storyteller',
    'strength',
    'studded',
    'stylist',
    'subliminal',
    'suction',
    'suds',
    'suitcase',
    'sulfur',
    'summertime',
    'supper',
    'survival',
    'suspend',
    'suture',
    'swarm',
    'sweat',
    'swig',
    'swoon',
    'symbiotic',
    'synchronize',
    'system',
    'table',
    'tactical',
    'tadpole',
    'tag',
    'tail',
    'takeaway',
    'tall',
    'tambourine',
    'tantalizing',
    'target',
    'tassel',
    'tattoo',
    'tavern',
    'taxonomy',
    'teacher',
    'technician',
    'teddy',
    'teeth',
    'telephone',
    'temporary',
    'tender',
    'terminus',
    'tesseract',
    'thank',
    'thesis',
    'thick',
    'thoughtful',
    'threshold',
    'thunder',
    'tickle',
    'tiebreaker',
    'tile',
    'timely',
    'tinker',
    'tip',
    'titan',
    'toadstool',
    'tofu',
    'together',
    'toil',
    'token',
    'tolerance',
    'tomorrow',
    'toolbox',
    'topology',
    'tortoise',
    'total',
    'touching',
    'toy',
    'trans',
    'treetop',
    'triangle',
    'trough',
    'truth',
    'tube',
    'tugging',
    'tulip',
    'tumble',
    'turing',
    'tusk',
    'tuxedo',
    'tweed',
    'twilight',
    'typewriter',
    'tyrannosaurus',
    'tzatziki',
    'ubiquitous',
    'ukulele',
    'ultimate',
    'umbrella',
    'unanimous',
    'unbowed',
    'uncle',
    'underground',
    'unexpected',
    'ungrateful',
    'unholy',
    'unite',
    'uno',
    'unprecedented',
    'unravel',
    'unscrew',
    'until',
    'upbringing',
    'upgrade',
    'uphold',
    'uppermost',
    'upstage',
    'uptime',
    'uranus',
    'urchin',
    'useful',
    'usurp',
    'vacation',
    'valkyrie',
    'vampire',
    'van',
    'vaporwave',
    'variety',
    'vase',
    'vat',
    'venerable',
    'versatile',
    'vet',
    'vicarious',
    'videotape',
    'vigor',
    'village',
    'vine',
    'violin',
    'virus',
    'vista',
    'vitamin',
    'vocally',
    'void',
    'volume',
    'voracious',
    'vote',
    'vouch',
    'waiting',
    'walking',
    'warning',
    'wasp',
    'waterfall',
    'wax',
    'waypoint',
    'weave',
    'website',
    'wedge',
    'weight',
    'well',
    'werewolf',
    'wheelbarrow',
    'whistle',
    'wick',
    'wield',
    'wiggle',
    'wilderness',
    'winged',
    'wispy',
    'witchcraft',
    'wolf',
    'woman',
    'wooly',
    'workout',
    'wreath',
    'wristwatch',
    'xenophilia',
    'yam',
    'yardstick',
    'yesterday',
    'yielding',
    'yoyo',
    'yogurt',
    'youth',
    'zany',
    'zero',
    'zigzag',
    'zipper',
    'zone',
    'zookeeper',
    'zucchini',
]

def fatal(s):
    print(s, file=sys.stderr)
    exit(1)

try:
    entropy = int(sys.argv[1])
except (IndexError, ValueError):
    fatal("""usage: {} <entropy>

Generates a passphrase with at least <entropy> bits of entropy based
on xenofem's diceware wordlist.  Outputs the passphrase on stdout (for
piping to a file or program), and a mnemonic on stderr (for the user
to view and memorize).

xenofem's list contains 6^4 words with unique 3-letter prefixes.  The
passphrase consists of the first 3 letters of each randomly-chosen
word, and the full words serve as a mnemonic.  Passphrases have about
10.3 bits of entropy per word in the mnemonic, 3.4 bits of entropy per
letter in the passphrase.

Randomness is generated by python's secrets library, which claims to
be suitable for cryptographic applications.  That's the full extent of
the security research I've done here.""".format(sys.argv[0]))

count = math.ceil(entropy / math.log2(len(wordlist)))

passphrase = []
for i in range(count):
    passphrase.append(secrets.choice(wordlist))

print(''.join(word[:3] for word in passphrase))
print(' '.join(passphrase), file=sys.stderr)
